<?php

namespace Drupal\contextualized_state_examples\Form;

use Drupal\contextualized_state\Event\ContextEvent;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Implements the DispatchExampleForm form controller.
 *
 * @see \Drupal\Core\Form\FormBase
 */
class DispatchExampleForm extends FormBase {

  /**
   * Build the simple form.
   *
   * A build form method constructs an array that defines how markup and
   * other form elements are included in an HTML form.
   *
   * @param array $form
   *   Default form array structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object containing current form state.
   *
   * @return array
   *   The render array defining the elements of the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $contextManager = \Drupal::service('contextualized_state.context_manager.service');
    $context = $contextManager->getContext('soccer_context_provider');

    $form['description'] = [
      '#type' => 'item',
      '#markup' => $this->t('This basic example shows a single text input element and a submit button'),
    ];

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#description' => $this->t('Title must be at least 5 characters in length.'),
      '#required' => TRUE,
    ];

    $form['context_values'] = [
      '#description' => $this->t('The context values'),
      '#type' => 'inline_template',
      '#template' => '<pre class="poc-title">The context values: <br>{{ context_values|json_encode(constant("JSON_PRETTY_PRINT")) }}</pre>',
      '#context' => [
        'context_values' => $context->getAll(),
      ],
    ];

    // Group submit handlers in an actions element with a key of "actions" so
    // that it gets styled correctly, and so that other modules may add actions
    // to the form. This is not required, but is convention.
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * Getter method for Form ID.
   *
   * The form ID is used in implementations of hook_form_alter() to allow other
   * modules to alter the render array built by this form controller. It must be
   * unique site wide. It normally starts with the providing module's name.
   *
   * @return string
   *   The unique ID of the form defined by this class.
   */
  public function getFormId(): string {
    return 'dispatch_example_form';
  }

  /**
   * Implements form validation.
   *
   * The validateForm method is the default method called to validate input on
   * a form.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $title = $form_state->getValue('title');
    if (strlen($title) < 5) {
      // Set an error for the form element with a key of "title".
      $form_state->setErrorByName('title', $this->t('The title must be at least 5 characters long.'));
    }
  }

  /**
   * Implements a form submit handler.
   *
   * The submitForm method is the default method called for any submit elements.
   *
   * @param array $form
   *   The render array of the currently built form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Object describing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /*
     * This would normally be replaced by code that actually does something
     * with the title.
     */
    $title = $form_state->getValue('title');

    $event = ContextEvent::from([
      'type' => 'football',
      'title' => $title,
      'campaign_id' => 21,
      'campaign_brand' => 'San Paolo',
      'game_type' => 'soccer',
      'player_id' => 23,
      'game_mode' => 'for_prize',
    ]);

    \Drupal::service('event_dispatcher')->dispatch(ContextEvent::ON_SET_CONTEXT, $event);

    $this->messenger()->addMessage($this->t('You specified a title of %title.', ['%title' => $title]));
  }

}
